@@ -74,6 +74,21 @@ NumericVector zdist_pdf(NumericVector x, double mu, double sigma, double a, doub
   NumericVector L = zdist_lpdf(x, mu, sigma, a, b);
   return exp(L);
 }
+// [[Rcpp::export(.zdist_cens_lpdf)]]
+double zdist_cens_lpdf(double lb, double ub, double mu, double sigma, double a, double b) {
+  
+  double l1  = R::pnorm( ub, mu, sigma, true, false) - R::pnorm( lb, mu, sigma, true, false);
+  double l2  = R::pnorm(-lb, mu, sigma, true, false) - R::pnorm(-ub, mu, sigma, true, false);
+
+  double l1_1  = R::pnorm(b, mu, sigma, true, false);
+  double l1_2  = R::pnorm(a, mu, sigma, true, false);
+  
+  double l2_1  = R::pnorm(-a, mu, sigma, true, false);
+  double l2_2  = R::pnorm(-b, mu, sigma, true, false);
+  
+  double L = log(l1 + l2) - log(l1_1-l1_2+l2_1-l2_2);
+  return L;
+}
 // [[Rcpp::export(.tdist_pdf)]]
 NumericVector tdist_pdf(NumericVector x, double mu, double df, double a, double b) {
   NumericVector L = tdist_lpdf(x, mu, df, a, b);
@@ -128,6 +143,25 @@ NumericMatrix compute_u_log_lik(NumericVector x, NumericVector mu, NumericVector
   }
   return ll;
 }
+NumericMatrix compute_u_log_lik_c(NumericVector x, NumericVector lb, NumericVector ub, NumericVector mu, NumericVector sigma, double a, double b){
+  
+  NumericMatrix ll_o(mu.size(), x.size());
+  NumericMatrix ll_c(mu.size(), lb.size());
+  
+  for(int k = 0; k < mu.size(); k++){
+    ll_o(k,_) = zdist_lpdf(x,mu[k],sigma[k],a,b);
+  }
+  
+  for(int k = 0; k < mu.size(); k++){
+    for(int i = 0; i < lb.size(); i++){
+      ll_c(k,i) = zdist_cens_lpdf(lb[i],ub[i],mu[k],sigma[k],a,b);  
+    }
+  }
+  
+  NumericMatrix ll = transpose(cbind(ll_o, ll_c));
+  
+  return ll;
+}
 NumericMatrix weight_u_log_lik(NumericMatrix ull, NumericVector theta){
   NumericMatrix ll(ull.nrow(), ull.ncol());
 
@@ -206,6 +240,7 @@ NumericVector select_x(NumericVector x, double a, double b){
   return x_new;
 }
 double get_prop_high(NumericVector x, double select_sig, double b){
+  
   double a = R::pnorm(select_sig/2, 0, 1, false, false);
   
   LogicalVector x_sig_true = x > a;
@@ -217,6 +252,19 @@ double get_prop_high(NumericVector x, double select_sig, double b){
   double prop_high = (1.0 * x_high.length()) / (1.0 * x_sig.length());
   return prop_high;
 }
+double get_prop_high_cens(NumericVector x, double select_sig, double b, int n_censored){
+  
+  double a = R::pnorm(select_sig/2, 0, 1, false, false);
+  
+  LogicalVector x_sig_true = x > a;
+  NumericVector x_sig      = x[x_sig_true];
+  
+  LogicalVector x_high_true = x > b;
+  NumericVector x_high      = x[x_high_true];
+  
+  double prop_high = (1.0 * x_high.length()) / (1.0 * (x_sig.length() + n_censored));
+  return prop_high;
+}
 
 
 // [[Rcpp::export(.zcurve_EM_fit_RCpp)]]
@@ -307,6 +355,50 @@ List zcurve_EM_fit_fast_RCpp(NumericVector x, NumericVector mu, NumericVector si
 
   return ret;
 }
+// [[Rcpp::export(.zcurve_EMc_fit_fast_RCpp)]]
+List zcurve_EMc_fit_fast_RCpp(NumericVector x, NumericVector lb, NumericVector ub,
+                              NumericVector mu, NumericVector sigma, NumericVector theta, double a, double b, double sig_level,
+                             int max_iter, double criterion) {
+  
+  double prop_high = get_prop_high_cens(x, sig_level, b, lb.size());
+  x = select_x(x,a,b);
+  
+  NumericMatrix log_lik (x.size(), mu.size());
+  NumericMatrix lik (x.size(), mu.size());
+  NumericVector l_row_sum (mu.size());
+  NumericMatrix p (x.size(), mu.size());
+  NumericVector Q (max_iter+1);
+  
+  int i= 0;
+  Q[i] = 0;
+  
+  
+  NumericMatrix u_log_lik =  compute_u_log_lik_c(x, lb, ub, mu, sigma, a, b);
+  do{
+    // E-step
+    log_lik   = weight_u_log_lik(u_log_lik, theta);
+    lik       = exp_matrix(log_lik);
+    l_row_sum = compute_l_row_sum(lik);
+    p         = compute_p(lik,l_row_sum);
+    
+    // M-step
+    theta = update_theta(p);
+    
+    Q[i+1] = sum(log(l_row_sum));
+    ++i;
+    
+  } while ((fabs(Q[i]-Q[i-1]) >= criterion) && (i < max_iter));
+  
+  List ret;
+  ret["iter"]      = i;
+  ret["Q"]         = Q[i];
+  ret["mu"]        = mu;
+  ret["weights"]   = theta;
+  ret["sigma"]     = sigma;
+  ret["prop_high"] = prop_high;
+  
+  return ret;
+}
 // [[Rcpp::export(.zcurve_EM_start_RCpp)]]
 List zcurve_EM_start_RCpp(NumericVector x, int type, int K,
                    NumericVector mu, NumericVector sigma,NumericVector mu_alpha, double mu_max,
@@ -511,6 +603,107 @@ List zcurve_EM_boot_fast_RCpp(NumericVector x,
 
   return ret;
 }
+
+// [[Rcpp::export(.zcurve_EMc_start_fast_RCpp)]]
+List zcurve_EMc_start_fast_RCpp(NumericVector x,  NumericVector lb, NumericVector ub, int K,
+                               NumericVector mu, NumericVector sigma, NumericVector mu_alpha, double mu_max,
+                               NumericVector theta_alpha,
+                               double a, double b, double sig_level,
+                               int fit_reps, int max_iter, double criterion) {
+  
+  NumericMatrix mu_reps        (fit_reps, K);
+  NumericMatrix weights_reps   (fit_reps, K);
+  IntegerVector iter_reps      (fit_reps);
+  NumericVector Q_reps         (fit_reps);
+  NumericVector prop_high_reps (fit_reps);
+  
+  NumericVector temp_theta  (K);
+  NumericVector temp_mu     (K);
+  
+  NumericVector new_mu      (K);
+  NumericVector new_weights (K);
+  int new_iter;
+  double new_Q;
+  double new_prop_high;
+  
+  for(int i = 0; i < fit_reps; i++){
+    temp_theta = dirichlet_rng(theta_alpha);
+    temp_mu    = mu;
+    
+    List temp_fit = zcurve_EMc_fit_fast_RCpp(x, lb, ub, temp_mu, sigma, temp_theta, a, b, sig_level,
+                                            max_iter, criterion);
+    
+    new_mu        = temp_fit["mu"];
+    new_weights   = temp_fit["weights"];
+    new_iter      = temp_fit["iter"];
+    new_Q         = temp_fit["Q"];
+    new_prop_high = temp_fit["prop_high"];
+    
+    mu_reps(i,_)      = new_mu;
+    weights_reps(i,_) = new_weights;
+    iter_reps[i]      = new_iter;
+    Q_reps[i]         = new_Q;
+    prop_high_reps[i] = new_prop_high;
+  }
+  
+  List ret;
+  ret["iter"]      = iter_reps;
+  ret["Q"]         = Q_reps;
+  ret["mu"]        = mu_reps;
+  ret["weights"]   = weights_reps;
+  ret["prop_high"] = prop_high_reps;
+  
+  return ret;
+}
+
+// [[Rcpp::export(.zcurve_EMc_boot_fast_RCpp)]]
+List zcurve_EMc_boot_fast_RCpp(NumericVector x, NumericVector lb, NumericVector ub,
+                              NumericVector mu, NumericVector sigma, NumericVector theta,
+                              double a, double b, double sig_level,
+                              int bootstrap, int max_iter, double criterion){
+  NumericMatrix mu_reps        (bootstrap, mu.size());
+  NumericMatrix weights_reps   (bootstrap, mu.size());
+  IntegerVector iter_reps      (bootstrap);
+  NumericVector Q_reps         (bootstrap);
+  NumericVector prop_high_reps (bootstrap);
+  
+  NumericVector temp_x;
+  
+  NumericVector new_mu      (mu.size());
+  NumericVector new_weights (mu.size());
+  int new_iter;
+  double new_Q;
+  double new_prop_high;
+  
+  for(int i = 0; i < bootstrap; i++){
+    temp_x = sample(x, x.size(), true);
+    
+    List temp_fit = zcurve_EMc_fit_fast_RCpp(temp_x, lb, ub, mu, sigma, theta, a, b, sig_level,
+                                            max_iter, criterion);
+    
+    new_mu        = temp_fit["mu"];
+    new_weights   = temp_fit["weights"];
+    new_iter      = temp_fit["iter"];
+    new_Q         = temp_fit["Q"];
+    new_prop_high = temp_fit["prop_high"];
+    
+    mu_reps(i,_)      = new_mu;
+    weights_reps(i,_) = new_weights;
+    iter_reps[i]      = new_iter;
+    Q_reps[i]         = new_Q;
+    prop_high_reps[i] = new_prop_high;
+  }
+  
+  List ret;
+  ret["iter"]      = iter_reps;
+  ret["Q"]         = Q_reps;
+  ret["mu"]        = mu_reps;
+  ret["weights"]   = weights_reps;
+  ret["prop_high"] = prop_high_reps;
+  
+  return ret;
+}
+
 /*
   double get_loglik(NumericVector x, NumericVector mu, NumericVector sigma, NumericVector theta, double a, double b) {
 
